# Drug Pathway Proximity Analysis

This repository contains a tool to compute network-based proximity between drug targets and genes shared between a disease and a biological pathway. It was developed as part of a pipeline for prioritizing drug candidates for repurposing, based on protein–protein interaction (PPI) networks and gene-level associations.

## Project Structure

```
drug_pathway_proximity/
├── __init__.py              # Package marker
├── __main__.py              # Entry point for the analysis
├── config.py                # Parameters (paths and constants)
├── data_loader.py           # Data preprocessing and filtering
├── graph_utils.py           # PPI graph construction and mapping
├── proximity_metrics.py     # Functions for distance/z-score calculations
├── analysis.py              # Triplet-level proximity computation
├── README.md                # This documentation
├── requirements.txt         # Python package dependencies
├── environment.yml          # Conda environment specification
```

## 📂 Input Data Description

Place the following files in the `../data/` folder:

- **triples.csv**  
  Contains candidate triplets of the form `(new_disease, pathway_id, drug_id)` extracted from DREBIOP dataset.\
  Columns: `new_disease`, `pathway_id`, `drug_id`

  The triplets were derived from the DREBIOP dataset, built upon previous research on the role of biological pathways in drug repurposing:

  - B. Otero-Carrasco, E. Ugarte Carro, L. Prieto-Santamaría, M. Diaz Uzquiano, J. P. Caraça-Valente Hernández, and A. Rodríguez-González, “Identifying patterns to uncover the importance of biological pathways on known drug repurposing scenarios,” *BMC Genomics*, vol. 25, no. 1, p. 43, Jan. 2024. [https://doi.org/10.1186/s12864-023-09913-1](https://doi.org/10.1186/s12864-023-09913-1)

  - B. Otero Carrasco, L. Prieto-Santamaría, and A. Rodríguez-González, “Exploring Innovative Approaches to Drug Repurposing: Emphasizing the Relevance of Biological Pathways,” in *RExPO24 Conference*, REPO4EU, Jul. 2024. [https://doi.org/10.58647/REXPO.24000039.v1](https://doi.org/10.58647/REXPO.24000039.v1)

- **new_disease_genes.csv**  
  Links disease identifiers to their associated genes, obtained from [DISNET](https://www.disnet.ctb.upm.es/) (which integrates this information from [DisGeNET](https://www.disgenet.org/)).\
  Columns: `disease_id_x`, `gene_disease_1`

- **pathways_genes.csv**  
  Maps each pathway to the genes it involves, obtained from [DISNET](https://www.disnet.ctb.upm.es/) (which integrates this information from [WikiPathways](https://www.wikipathways.org/)).\
  Columns: `pathway_id`, `gene_id`

- **target.csv**  
  Drug–target associations, including drug name, obtained from [DISNET](https://www.disnet.ctb.upm.es/) (which integrates this information from [DrugBank](https://go.drugbank.com/)).\
  Columns: `drug_id`, `drug_name`, `gene_id`

- **pro_pro.tsv**  
  Protein–protein interactions, obtained from [DISNET](https://www.disnet.ctb.upm.es/) (which integrates this information from [neXtProt](https://www.nextprot.org/)).\
  Columns: `prA`, `prB` (UniProt IDs)

- **gen_pro.tsv**  
  Mapping of gene identifiers (NCBI) to protein identifiers (UniProt Accession Numbers).\
  Columns: `gen`, `pro`

All identifiers are harmonized to operate over a unified PPI network using gene symbols.

---

### Entity Summary Per File

| File                   | Description                                 | # Rows | # Unique Entries |
|------------------------|---------------------------------------------|--------|------------------|
| `triples.csv`          | Disease–Pathway–Drug triplets               | 12,614 | 133 diseases / 48 pathways / 556 drugs |
| `new_disease_genes.csv`| Disease–gene associations                   |   308  | 133 diseases / 282 genes              |
| `pathways_genes.csv`   | Pathway–gene associations                   |   203  | 48 pathways / 173 genes               |
| `target.csv`           | Drug–target associations                    |  2,729 | 556 drugs / 684 genes                 |
| `pro_pro.tsv`          | Protein–protein interactions                | 370,267| 19,827 unique proteins (10,060 prA / 9,767 prB) |
| `gen_pro.tsv`          | Gene–protein mapping                        | 13,979 | 13,979 genes / 20,248 proteins        |

---

## Method Overview

This script implements the **network analysis module** of the drug repurposing strategy described in:

> **"Prioritization of Potential Drugs Through Pathway-Based Drug Repurposing and Network Proximity Analysis"**  
> To be presented at *CBMS 2025*.

The method builds on the principle that drugs whose targets are close—in the interactome—to biological processes disrupted in a disease are more likely to have therapeutic potential.

The network-based proximity metric is inspired by:

> Guney, E., Menche, J., Vidal, M. & Barabási, A.-L. *Network-based in silico drug efficacy screening*. Nature Communications **7**, 10331 (2016).  
> [https://www.nature.com/articles/ncomms10331](https://www.nature.com/articles/ncomms10331)

---

## Processing Pipeline

### 1. Data Preprocessing

- Start from curated Disease–Pathway–Drug triplets.
- Remove any triplet where a gene is shared between the disease, pathway, and drug.
- Retain only genes present in the PPI network.
- Filter to keep triplets with at least one gene in the disease-pathway and drug sets.

### 2. PPI Network Construction

- Build the network from neXtProt protein–protein interactions.
- Convert all protein identifiers to NCBI gene symbols.
- Construct an undirected graph of genes and their known physical interactions.

---

## Network-Based Proximity Computation

For each Disease–Pathway–Drug triplet:

1. Define:
   - **S**: genes shared between the disease and pathway
   - **T**: drug target genes

2. Compute:
   - **Minimum SPL**: shortest path between any gene in S and any gene in T
   - **Average SPL (d_c)**: average of shortest distances from each target gene (T) to the closest pathway gene (S)
   - **Z-score**: compares the observed average SPL with a distribution of SPLs from random sets preserving degree

3. Report:
   - Proximity metrics
   - Genes in shortest paths
   - Closest interacting gene pairs

---

## Running the Analysis

From the project root directory:

```bash
python -m drug_pathway_proximity
```

### 🔧 Environment Setup

You can install required dependencies using pip:

```bash
pip install -r requirements.txt
```

Or create a Conda environment with:

```bash
conda env create -f environment.yml
conda activate drug-pathway-proximity
```

---

## Output

The results will be saved to:

```
../results/drug_pathway_proximity_results.csv
```

Each row includes:
- Disease, pathway, and drug identifiers
- Number of mapped genes per set
- Proximity metrics: min SPL, avg SPL, Z-score
- Shortest path intermediates and gene connections

---

## Authors

B. Otero-Carrasco, L. Prieto-Santamaría, and A. Rodríguez-González \
MEDAL · [https://medal.ctb.upm.es](https://medal.ctb.upm.es)  
CTB · [https://ctb.upm.es](https://ctb.upm.es) 

### Main developer of the module
[Lucía Prieto Santamaría](https://luciaprietosantamaria.es/)
Assistant Professor, [Universidad Politécnica de Madrid (UPM)](https://www.upm.es/)  
📧 [lucia.prieto.santamaria@upm.es](lucia.prieto.santamaria@upm.es)