import pandas as pd

def load_and_prepare_data(data_dir):
    """
    Loads and prepares triples, drug-targets, and disease-pathway-gene associations.
    Implements pre-analysis filtering to remove triples where disease, pathway and drug share a gene.
    Filters to ensure all mapped genes are present in the PPI network.

    Args:
        data_dir (str): Path to the data directory.

    Returns:
        tuple: filtered_triples, dru_pro_filtered, dis_pat_filtered DataFrames
    """
    print("Loading and processing input files...")

    triples = pd.read_csv(f'{data_dir}/triples.csv').iloc[:, 1:]
    triples.rename(columns={'new_disease': 'disease_id'}, inplace=True)
    print(f"Loaded {len(triples)} triples.")

    new_disease = pd.read_csv(f'{data_dir}/new_disease_genes.csv').iloc[:, 1:]
    new_disease.rename(columns={'disease_id_x': 'disease_id', 'gene_disease_1': 'gene_id'}, inplace=True)
    print(f"Loaded {len(new_disease)} new_disease gene mappings.")

    pathways = pd.read_csv(f'{data_dir}/pathways_genes.csv').iloc[:, 1:]
    print(f"Loaded {len(pathways)} pathway-gene associations.")

    target = pd.read_csv(f'{data_dir}/target.csv').iloc[:, 1:]
    target = target[['drug_id', 'drug_name', 'gene_id']].dropna().drop_duplicates().reset_index(drop=True)
    print(f"Loaded {len(target)} valid drug-gene associations.")

    dru_pro = target[['drug_id', 'gene_id']].drop_duplicates().reset_index(drop=True)

    # Filter: remove triples where disease-pathway-drug share the same gene
    print("Removing overlapping triples with shared genes...")
    subset_to_remove = (
        triples
        .merge(new_disease, on='disease_id')
        .merge(pathways, on=['pathway_id', 'gene_id'])
        .merge(dru_pro, on=['drug_id', 'gene_id'])
        [['disease_id', 'pathway_id', 'drug_id']]
        .drop_duplicates()
    )

    filtered_triples = (
        triples
        .merge(subset_to_remove, on=['disease_id', 'pathway_id', 'drug_id'], how='left', indicator=True)
        .query('_merge == "left_only"')
        .drop(columns=['_merge'])
        .reset_index(drop=True)
    )
    print(f"Filtered down to {len(filtered_triples)} triples after overlap removal.")

    # Construct dis_pat from filtered_triples
    dis_pat = (
        filtered_triples[['disease_id', 'pathway_id']]
        .drop_duplicates()
        .merge(new_disease, on='disease_id')
        .merge(pathways, on=['pathway_id', 'gene_id'])
        [['disease_id', 'pathway_id', 'gene_id']]
        .drop_duplicates()
        .reset_index(drop=True)
    )

    # Load PPI and gene-protein mapping
    ppi = pd.read_csv(f'{data_dir}/pro_pro.tsv', sep='\t')
    gen_pro = pd.read_csv(f'{data_dir}/gen_pro.tsv', sep='\t')

    ppi = (
        ppi.merge(gen_pro, left_on='prA', right_on='pro').drop('pro', axis=1).rename(columns={'gen': 'genA'})
           .merge(gen_pro, left_on='prB', right_on='pro').drop('pro', axis=1).rename(columns={'gen': 'genB'})
           [['genA', 'genB']]
           .drop_duplicates()
           .reset_index(drop=True)
    )
    all_genes_ppi = pd.unique(ppi[['genA', 'genB']].values.ravel())

    dis_pat_filtered = dis_pat[dis_pat.gene_id.isin(all_genes_ppi)].reset_index(drop=True)
    dru_pro_filtered = dru_pro[dru_pro.gene_id.isin(all_genes_ppi)].reset_index(drop=True)

    # Keep only triples where both sides have genes in the PPI
    final_triples = []
    for _, row in filtered_triples.iterrows():
        dis_genes = dis_pat_filtered.query("disease_id == @row.disease_id and pathway_id == @row.pathway_id")["gene_id"]
        drug_genes = dru_pro_filtered.query("drug_id == @row.drug_id")["gene_id"]
        if not dis_genes.empty and not drug_genes.empty:
            final_triples.append(row)

    final_triples = pd.DataFrame(final_triples).reset_index(drop=True)
    print(f"Final set contains {len(final_triples)} valid triples with genes in PPI.")

    return final_triples, dru_pro_filtered, dis_pat_filtered