import numpy as np
import random
import graph_tool.all as gt


def get_min_and_avg_spls_A_B(A, B, shortest_dict):
    """
    Computes the minimum and average shortest path lengths (SPLs) between 
    two sets of nodes A and B in the interactome.

    Parameters:
    - A (iterable of int): source gene node IDs (e.g., disease-pathway genes)
    - B (iterable of int): target gene node IDs (e.g., drug target genes)
    - shortest_dict (dict): precomputed shortest path matrix per vertex

    Returns:
    - (float, float): minimum SPL, average SPL
    """
    A = set(A)
    B = set(B)
    bad_values = (float('inf'), np.nan, None, 2147483647)
    spls = []
    for a in A:
        spl = float('inf')
        for b in B:
            if a != b:
                shortest = shortest_dict[a][b]
                if shortest not in bad_values:
                    spl = min(shortest, spl)
            else:
                spl = 0
        spls.append(spl)
    if len(spls) > 0:
        return np.min(spls), np.mean(spls)
    return -1, -1


def get_edged_A_B(A, B, g, id2gene):
    """
    Identifies pairs of nodes from sets A and B that are connected 
    via shortest paths of minimal length in the graph `g`.

    For each pair (a in A, b in B), it computes the shortest path length. 
    It keeps track of the minimal length observed, and returns all (a, b) pairs 
    that meet that minimal length.

    Parameters:
    - A (iterable of int): source gene node indices (e.g., disease-pathway genes)
    - B (iterable of int): target gene node indices (e.g., drug target genes)
    - g (graph_tool.Graph): undirected PPI network graph

    Returns:
    - dict:
        - 'A_prots': list of node indices from A involved in minimal paths
        - 'B_prots': list of node indices from B involved in minimal paths
        - 'paths': list of [a, b] node index pairs with minimal shortest path
    """
    A, B = set(A), set(B)
    A_prots, B_prots, paths = set(), set(), []
    min_length = float('inf')
    
    for a in A:
        for b in B:
            if a != b:
                path, _ = gt.shortest_path(g, g.vertex(a), g.vertex(b))
                if path:
                    path_length = len(path) - 1
                    if path_length < min_length:
                        min_length = path_length
                        paths = [[a, b]]
                        A_prots, B_prots = {a}, {b}
                    elif path_length == min_length:
                        paths.append([a, b])
                        A_prots.update([a])
                        B_prots.update([b])
    
    genes_in_paths = list()
    for path in paths:
        path_genes = []
        for protein in path:
            path_genes.append(id2gene[protein])
        genes_in_paths.append(path_genes)
    
    return {
        'A_prots': [id2gene[v] for v in A_prots], 
        'B_prots': [id2gene[v] for v in B_prots], 
        'paths': genes_in_paths
    }


def get_binning(net, ids, bin_size=100):
    """
    Groups the graph nodes by their degrees into bins of approximately `bin_size` size.

    Parameters:
    - net (graph_tool.Graph): the interactome graph
    - ids (VertexPropertyMap): maps internal graph vertices to external gene IDs
    - bin_size (int): target number of nodes per bin

    Returns:
    - lower (list of int): lower bound of degree for each bin
    - upper (list of int): upper bound of degree for each bin
    - nodes (list of list): binned external gene IDs per degree range
    """
    deg = net.degree_property_map('out')
    degree2nodes = {}
    for v in net.iter_vertices():
        degree2nodes.setdefault(deg[v], []).append(ids[v])
    degrees = sorted(degree2nodes)
    lower, upper, nodes, counter, cumnodes, l = [], [], [], 0, [], 0
    for d in degrees:
        counter += len(degree2nodes[d])
        cumnodes.extend(degree2nodes[d])
        if counter < bin_size:
            continue
        u = d
        lower.append(l)
        upper.append(u)
        nodes.append(cumnodes)
        l, cumnodes, counter = u, [], 0
    if counter:
        upper[-1] = d
        nodes[-1].extend(cumnodes)
    upper[-1] += 1
    return lower, upper, nodes



def dictionary_from_binning(lower, upper, values, gene2id):
    """
    Builds a degree-to-vertex-index dictionary using binning information.

    Parameters:
    - lower, upper (list of int): bin ranges for node degrees
    - values (list of list): binned gene IDs per range
    - gene2id (dict): maps gene symbols to graph vertex IDs

    Returns:
    - dict: degree → list of vertex indices
    """
    degree_dict = {}
    for l, u, nodes in zip(lower, upper, values):
        for degree in range(l, u):
            nodes_tr = [gene2id[a] for a in nodes]
            degree_dict[degree] = nodes_tr
    return degree_dict



def sample_preserving_degrees(net, S, bucket):
    """
    Randomly sample nodes with the same degree distribution as those in `S`.

    Parameters:
    - net (graph_tool.Graph): graph to sample from
    - S (iterable): set of source node indices
    - bucket (dict): mapping of degree to available node indices

    Returns:
    - set: sampled node indices from `net` preserving the degree profile of `S`
    """
    sampled_nodes = set()
    deg = net.degree_property_map('out')
    for node in S:
        degree = deg[node]
        available_nodes = bucket[degree]
        while True:
            chosen_node = random.choice(available_nodes)
            if chosen_node not in sampled_nodes:
                sampled_nodes.add(chosen_node)
                break
    return sampled_nodes



def get_proximity_z_score(net, T, S, shortest_dict, n_iter, bin_size, ids, gene2id):
    """
    Calculates a proximity Z-score between sets T and S by comparing the observed
    average shortest path length to a random distribution that preserves degree.

    Parameters:
    - net (graph_tool.Graph): the interactome graph
    - T (set of int): drug target node indices
    - S (set of int): disease-pathway gene node indices
    - shortest_dict (dict): precomputed shortest distances between all nodes
    - n_iter (int): number of random iterations
    - bin_size (int): minimum number of nodes per bin
    - ids: vertex ID map
    - gene2id (dict): map from gene symbol to graph node index

    Returns:
    - float: proximity Z-score between the two sets
    """
    lower, upper, nodes = get_binning(net, ids, bin_size=bin_size)
    bucket = dictionary_from_binning(lower, upper, nodes, gene2id)
    d_c = get_min_and_avg_spls_A_B(T, S, shortest_dict)[1]
    distribution = []
    for _ in range(n_iter):
        ran_T  = sample_preserving_degrees(net, T, bucket)
        ran_S  = sample_preserving_degrees(net, S, bucket)
        ran_d_c = get_min_and_avg_spls_A_B(ran_T, ran_S, shortest_dict)[1]
        distribution.append(ran_d_c)
    distribution = [x for x in distribution if x not in (float('inf'), np.nan, None, 2147483647)]
    mu, sigma = np.mean(distribution), np.std(distribution)
    return (d_c - mu) / sigma



def get_shortest_paths_A_B(A, B, g, id2gene):
    """
    Return the intermediate proteins in the shortest paths between nodes sets A and B.
    If the shortest path has no intermediates (i.e. length 1), nothing is added.

    Parameters:
    - A (set of int): source vertex indices
    - B (set of int): target vertex indices
    - g (graph_tool.Graph): undirected interactome

    Returns:
    - list of gene names corresponding to intermediate nodes
    """
    A = set(A)
    B = set(B)
    proteins_in_paths = set()
    min_length = float('inf')

    for a in A:
        for b in B:
            if a != b:
                path, _ = gt.shortest_path(g, g.vertex(a), g.vertex(b))
                if path:
                    path_length = len(path) - 1
                    if path_length < min_length and path_length > 1:
                        proteins_in_paths = set(int(v) for v in path[1:-1])
                        min_length = path_length
                    elif path_length == min_length and path_length > 1:
                        proteins_in_paths.update(int(v) for v in path[1:-1])

    return [id2gene[v] for v in proteins_in_paths]