"""
Tripartite network visualization: Drug – Gene – Disease

Author: Lucía Prieto Santamaría
Email: lucia.prieto.santamaria@upm.es

This script was developed as part of the paper:
"Prioritization of Potential Drugs Through Pathway-Based Drug Repurposing and Network Proximity Analysis",
accepted at the IEEE 38th International Symposium on Computer-Based Medical Systems (CBMS 2025).

It generates and exports a tripartite network figure showing:
- Drug → Gene regulations (up/down),
- Gene – Disease associations (undirected),
- Node types (drug, gene, disease) with a color-coded legend.

Outputs: PNG and SVG versions of the figure.
"""

import pandas as pd
import networkx as nx
from collections import defaultdict

import matplotlib.pyplot as plt
import matplotlib.lines as mlines
import matplotlib.patches as mpatches
from matplotlib.legend_handler import HandlerPatch


class HandlerArrow(HandlerPatch):
    """Custom legend handler to display arrows with heads on the right side."""
    def create_artists(self, legend, orig_handle,
                       xdescent, ydescent, width, height, fontsize, trans):
        arrow = mpatches.FancyArrowPatch((0, 0), (1, 0),
                                         arrowstyle='-|>',
                                         mutation_scale=15,
                                         color=orig_handle.get_edgecolor(),
                                         lw=orig_handle.get_linewidth(),
                                         linestyle=orig_handle.get_linestyle(),
                                         transform=trans)
        arrow.set_positions((xdescent, ydescent + height / 2),
                            (xdescent + width, ydescent + height / 2))
        return [arrow]


def plot_tripartite(df: pd.DataFrame, output_prefix: str):
    """Plot a tripartite drug–gene–disease network from dataframe and save it to disk."""
    
    # Build graphs
    G_directed = nx.MultiDiGraph()
    G_undirected = nx.Graph()

    for _, row in df.iterrows():
        drug = f"drug::{row['pert_iname']}"
        gene = f"gene::{row['pr_gene_symbol']}"
        disease = f"disease::{row['disease_name']}"
        G_directed.add_edge(drug, gene, regulation=row['regulation'])
        G_undirected.add_edge(gene, disease)

    H = nx.DiGraph()
    H.add_edges_from(G_directed.edges(keys=False))
    H.add_edges_from(G_undirected.edges())

    # Group nodes
    drug_nodes = [n for n in H.nodes if n.startswith("drug::")]
    gene_nodes = [n for n in H.nodes if n.startswith("gene::")]
    disease_nodes = [n for n in H.nodes if n.startswith("disease::")]

    # Sort by degree
    gene_nodes_sorted = sorted(gene_nodes, key=lambda n: -H.degree[n])
    gene_pos = {n: -i for i, n in enumerate(gene_nodes_sorted)}

    def order_by_gene_alignment(source_nodes, edges):
        node_y = {}
        for node in source_nodes:
            connected_genes = [v for u, v, _ in edges if u == node]
            y_vals = [gene_pos.get(g, 0) for g in connected_genes]
            node_y[node] = sum(y_vals)/len(y_vals) if y_vals else 0
        return sorted(source_nodes, key=lambda n: -node_y[n])

    drug_nodes_sorted = order_by_gene_alignment(drug_nodes, G_directed.edges(keys=True))
    disease_nodes_sorted = order_by_gene_alignment(disease_nodes, [(v, u, None) for u, v in G_undirected.edges()])

    # Layout
    spacing = 1.5
    pos = {}
    for i, n in enumerate(drug_nodes_sorted):
        pos[n] = (0, -i * spacing)
    for i, n in enumerate(gene_nodes_sorted):
        pos[n] = (1, -i * spacing - 1.5)
    for i, n in enumerate(disease_nodes_sorted):
        pos[n] = (2, -i * spacing + 2)

    color_map = {'drug': '#1b9e77', 'gene': '#d95f02', 'disease': '#7570b3'}
    node_colors = [
        color_map['drug'] if n.startswith("drug::") else
        color_map['gene'] if n.startswith("gene::") else
        color_map['disease']
        for n in H.nodes
    ]

    regulations_per_pair = defaultdict(set)
    for u, v, data in G_directed.edges(data=True):
        regulations_per_pair[(u, v)].add(data['regulation'])

    # Plot
    plt.figure(figsize=(8, 10))
    nx.draw_networkx_nodes(H, pos, node_color=node_colors, node_size=500)

    for u, v, data in G_directed.edges(data=True):
        reg_type = data['regulation']
        color = '#e41a1c' if reg_type == 'up' else '#377eb8' if reg_type == 'down' else 'gray'

        if len(regulations_per_pair[(u, v)]) == 1:
            nx.draw_networkx_edges(
                H, pos, edgelist=[(u, v)],
                edge_color=color, arrows=True, arrowsize=15,
                arrowstyle='-|>'
            )
        else:
            rad = 0.05 if reg_type == 'up' else -0.05
            nx.draw_networkx_edges(
                H, pos, edgelist=[(u, v)],
                edge_color=color, arrows=True, arrowsize=15,
                connectionstyle=f'arc3,rad={rad}', arrowstyle='-|>'
            )

    nx.draw_networkx_edges(H, pos, edgelist=G_undirected.edges(), edge_color='lightgray', arrows=False)

    # Labels
    labels = {n: n.split("::")[1] for n in H.nodes}
    for node, (x, y) in pos.items():
        plt.text(x, y - 0.5, labels[node], fontsize=9, ha='center', va='top')

    # Legend
    legend_elements = [
        mlines.Line2D([], [], color='white', marker='o', markersize=12,
                      markerfacecolor=color_map['drug'], label='Drug'),
        mlines.Line2D([], [], color='white', marker='o', markersize=12,
                      markerfacecolor=color_map['gene'], label='Gene'),
        mlines.Line2D([], [], color='white', marker='o', markersize=12,
                      markerfacecolor=color_map['disease'], label='Disease'),
        mpatches.FancyArrowPatch((0, 0), (1, 0), connectionstyle='arc3,rad=0',
                                 arrowstyle='-|>', color='#e41a1c', lw=2, label='Up-regulation'),
        mpatches.FancyArrowPatch((0, 0), (1, 0), connectionstyle='arc3,rad=0',
                                 arrowstyle='-|>', color='#377eb8', lw=2, label='Down-regulation'),
        mlines.Line2D([], [], color='lightgray', linewidth=2, label='Gene–Disease association')
    ]

    plt.legend(handles=legend_elements,
               handler_map={mpatches.FancyArrowPatch: HandlerArrow()},
               loc='lower center', bbox_to_anchor=(0.5, 0.9),
               ncol=2, fontsize=9, frameon=False)

    plt.axis('off')
    plt.tight_layout()

    # Save to files
    plt.savefig(f'{output_prefix}.svg', format='svg', dpi=700, bbox_inches='tight')
    plt.savefig(f'{output_prefix}.png', format='png', dpi=600, bbox_inches='tight')
    plt.show()


def main():
    # Load and preprocess data
    df = pd.read_excel('../data/tripartite_network.xlsx')
    df = df[['pert_iname', 'pr_gene_symbol', 'disease_name', 'regulation']].drop_duplicates().reset_index(drop=True)

    # Generate and export figure
    plot_tripartite(df, output_prefix='../figures/tripartite_network')


if __name__ == '__main__':
    main()